@@ -1,4 +1,4 @@
-# Copyright (C) 2021 DigeeX
+# Copyright (C) 2022 DigeeX
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 """Flow class holding the information exchanged between server and client.
 """
 
-import logging
 from typing import List, Optional
 
 import requests
@@ -31,23 +30,21 @@ from raider.user import User
 class Flow:
     """Class dealing with the information exchange from HTTP communication.
 
-    A Flow object in Raider defines all the information about one single
-    HTTP information exchange. It has a ``name``, contains one
-    ``request``, the ``response``, the ``outputs`` that needs to be
-    extracted from the response, and a list of ``operations`` to be run
-    when the exchange is over.
+    A Flow object in Raider defines all the information about one
+    single HTTP information exchange. It contains one ``request``, the
+    ``response``, the ``outputs`` that needs to be extracted from the
+    response, and a list of ``operations`` to be run when the exchange
+    is over.
 
-    Flow objects are used as states in the :class:`Authentication
-    <raider.authentication.Authentication>` class to define the
-    authentication process as a finite state machine.
+    Use this only when working with requests that *DON't* change the
+    authentication state.  It's used in the :class:`Functions
+    <raider.functions.Functions>` class to run arbitrary actions when
+    it doesn't affect the authentication state.
 
-    It's also used in the :class:`Functions
-    <raider.functions.Functions>` class to run arbitrary actions when it
-    doesn't affect the authentication state.
+    When the authentication state changes, use :class:`AuthFlow
+    <raider.flow.AuthFlow>` objects.
 
     Attributes:
-      name:
-        A string used as a unique identifier for the defined Flow.
       request:
         A :class:`Request <raider.request.Request>` object detailing the
         HTTP request with its elements.
@@ -65,17 +62,13 @@ class Flow:
         outputs are extracted. Should contain a :class:`NextStage
         <raider.operations.NextStage>` operation if another Flow is
         expected.
-      logger:
-        A :class:`logging.RootLogger` object used for debugging.
-
     """
 
     def __init__(
         self,
-        name: str,
         request: Request,
-        outputs: List[Plugin] = None,
-        operations: List[Operation] = None,
+        outputs: Optional[List[Plugin]] = None,
+        operations: Optional[List[Operation]] = None,
     ) -> None:
         """Initializes the Flow object.
 
@@ -83,8 +76,6 @@ class Flow:
         to be extracted, and the operations to be run upon completion.
 
         Args:
-          name:
-            A string with a unique identifier for this Flow.
           request:
             A Request object associated with this Flow.
           outputs:
@@ -95,15 +86,11 @@ class Flow:
             received.
 
         """
-        self.name = name
-
         self.outputs = outputs
         self.operations = operations
 
         self.request = request
-        self.response: requests.models.Response = None
-
-        self.logger = logging.getLogger(self.name)
+        self.response: Optional[requests.models.Response] = None
 
     def execute(self, user: User, config: Config) -> None:
         """Sends the request and extracts the outputs.
@@ -128,6 +115,8 @@ class Flow:
             for output in self.outputs:
                 if output.needs_response:
                     output.extract_value_from_response(self.response)
+                    if output.name_not_known_in_advance:
+                        output.extract_name_from_response(self.response)
                 elif output.depends_on_other_plugins:
                     for item in output.plugins:
                         item.get_value(user.to_dict())
@@ -166,3 +155,51 @@ class Flow:
                     break
 
         return next_stage
+
+
+class AuthFlow(Flow):
+    """Class dealing with the authentication Flows.
+
+    It inherits from :class:`Flow <raider.flow.Flow>` so it contains
+    one ``request``, the ``response``, the ``outputs`` that needs to
+    be extracted from the response, and a list of ``operations`` to be
+    run when the exchange is over.
+
+    Use this only when working with requests that *DO* change the
+    authentication state.  It's used in the :class:`Authentication
+    <raider.authentication.Authentication>` class to run actions that
+    affect the authentication state.
+
+    When the authentication state doesn't change, use :class:`Flow
+    <raider.flow.Flow>` objects.
+
+    Attributes:
+      request:
+        A :class:`Request <raider.request.Request>` object detailing the
+        HTTP request with its elements.
+      response:
+        A :class:`requests.model.Response` object. It's empty until the
+        request is sent. When the HTTP response arrives, it's stored
+        here.
+      outputs:
+        A list of :class:`Plugin <raider.plugins.Plugin>` objects
+        detailing the pieces of information to be extracted from the
+        response. Those will be later available for other Flow objects.
+      operations:
+        A list of :class:`Operation <raider.operations.Operation>`
+        objects to be executed after the response is received and
+        outputs are extracted. Should contain a :class:`NextStage
+        <raider.operations.NextStage>` operation if another Flow is
+        expected.
+
+    """
+    def __init__(
+        self,
+        request: Request,
+        outputs: Optional[List[Plugin]] = None,
+        operations: Optional[List[Operation]] = None,
+    ) -> None:
+
+        super().__init__(
+            request=request, outputs=outputs, operations=operations
+        )
