@@ -1,4 +1,4 @@
-# Copyright (C) 2021 DigeeX
+# Copyright (C) 2022 DigeeX
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 
 import logging
 import sys
-from typing import List, Optional
+from typing import Dict, Optional
 
 from raider.config import Config
 from raider.flow import Flow
@@ -39,7 +39,7 @@ class Functions:
 
     """
 
-    def __init__(self, functions: List[Flow]) -> None:
+    def __init__(self, functions: Dict[str, Flow]) -> None:
         """Initializes the Functions object.
 
         Args:
@@ -50,29 +50,22 @@ class Functions:
         """
         self.functions = functions
 
-    def get_function_by_name(self, name: str) -> Optional[Flow]:
-        """Gets the function given its name.
+    def __getitem__(self, key: str) -> Optional[Flow]:
+        if key not in self.functions.keys():
+            return None
+        return self.functions[key]
 
-        Tries to find the Flow object with the given name, and returns
-        it if it's found, otherwise returns None.
+    def get_flow_index(self, name: str) -> Optional[int]:
+        """Returns the index of Flow in the Functions array."""
+        keys = list(self.functions.keys())
 
-        Args:
-          name:
-            A string with the unique identifier of the function as
-            defined in the Flow.
+        if not name or name not in keys:
+            return -1
 
-        Returns:
-          A Flow object associated with the name, or None if no such
-          function has been found.
+        return keys.index(name)
 
-        """
-        for function in self.functions:
-            if function.name == name:
-                return function
-        return None
-
-    def run(self, name: str, user: User, config: Config) -> None:
-        """Runs a Function.
+    def run_flow(self, name: str, user: User, config: Config) -> Optional[str]:
+        """Runs a single Flow.
 
         Executes the given function, in the context of the specified
         user, and applies the global Raider configuration.
@@ -88,7 +81,7 @@ class Functions:
 
         """
         logging.info("Running function %s", name)
-        function = self.get_function_by_name(name)
+        function = self.functions[name]
         if function:
             function.execute(user, config)
             if function.outputs:
@@ -96,8 +89,30 @@ class Functions:
                     if item.value:
                         user.set_data(item)
 
-            function.run_operations()
+            next_stage = function.run_operations()
+            return next_stage
 
-        else:
-            logging.critical("Function %s not defined. Cannot continue", name)
-            sys.exit()
+        logging.critical("Function %s not defined. Cannot continue", name)
+        sys.exit()
+
+    def run_chain(self, name: str, user: User, config: Config) -> None:
+        """Runs a Function, and follows the NextStage.
+
+        Executes the given function, in the context of the specified
+        user, and applies the global Raider configuration. Runs the next
+        defined stage.
+
+
+        Args:
+          name:
+            A string with the name of the function to run.
+          user:
+            A User object containing all the data needed to run the
+            function in this user's context.
+          config:
+            A Config object with the global Raider configuration.
+
+        """
+        next_stage = name
+        while next_stage:
+            next_stage = self.run_flow(next_stage, user, config) or ""
