@@ -1,4 +1,4 @@
-# Copyright (C) 2021 DigeeX
+# Copyright (C) 2022 DigeeX
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -381,18 +381,18 @@ class Save(Operation):
 
         """
         if self.will_append:
-            mode = "a"
+            mode = "ab"
         else:
-            mode = "w"
+            mode = "wb"
 
         with open(self.filename, mode) as outfile:
             if isinstance(content, requests.models.Response):
-                outfile.write(content.text)
+                outfile.write(content.content)
             elif isinstance(content, Plugin):
-                outfile.write(content.value)
+                outfile.write(bytes(content.value, "utf-8"))
             else:
-                outfile.write(content)
-            outfile.write("\n")
+                outfile.write(bytes(content, "utf-8"))
+            outfile.write(b"\n")
 
     @classmethod
     def append(cls, filename: str, plugin: Plugin) -> "Save":
