@@ -1,4 +1,4 @@
-# Copyright (C) 2021 DigeeX
+# Copyright (C) 2022 DigeeX
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,9 +21,9 @@ import sys
 from typing import Optional
 
 from raider.application import Application
-from raider.attacks import Fuzz
 from raider.authentication import Authentication
 from raider.config import Config
+from raider.fuzzing import Fuzz
 from raider.plugins.common import Plugin
 from raider.user import User
 
@@ -118,7 +118,28 @@ class Raider:
         if self.session_loaded:
             self.fix_function_plugins(function)
 
-        self.functions.run(function, self.user, self.config)
+        self.functions.run_flow(function, self.user, self.config)
+
+    def run_chain(self, function: str) -> None:
+        """Runs a function, and follows the NextStage Operations.
+
+        With the selected application and user run the function from the
+        argument.
+
+        Args:
+          function:
+            A string with the function identifier as defined in
+            "_functions" variable.
+
+        """
+        if not hasattr(self, "functions"):
+            logging.critical("No functions defined. Cannot continue.")
+            sys.exit()
+
+        if self.session_loaded:
+            self.fix_function_plugins(function)
+
+        self.functions.run_chain(function, self.user, self.config)
 
     def fuzz(
         self,
@@ -142,9 +163,9 @@ class Raider:
 
         """
         is_authentication = False
-        flow = self.functions.get_function_by_name(flow_name)
+        flow = self.functions[flow_name]
         if not flow:
-            flow = self.authentication.get_stage_by_name(flow_name)
+            flow = self.authentication[flow_name]
             is_authentication = True
         if flow:
             if self.session_loaded:
@@ -181,7 +202,7 @@ class Raider:
         extracted data instead of extracting it again.
 
         """
-        flow = self.functions.get_function_by_name(function)
+        flow = self.functions[function]
         if not flow:
             logging.critical(
                 "Function %s not found. Cannot continue.", function
@@ -194,7 +215,7 @@ class Raider:
             for plugin in inputs.values():
                 # Reset plugin flags, and get the values from userdata
                 plugin.flags = Plugin.NEEDS_USERDATA
-                plugin.function = plugin.extract_from_userdata
+                plugin.function = plugin.extract_value_from_userdata
 
     @property
     def authentication(self) -> Authentication:
