@@ -1,4 +1,4 @@
-# Copyright (C) 2021 DigeeX
+# Copyright (C) 2022 DigeeX
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,13 +22,12 @@ import sys
 import urllib
 from copy import deepcopy
 from typing import Any, Dict, List, Optional, Union
-from urllib import parse
 
 import requests
 from urllib3.exceptions import InsecureRequestWarning
 
 from raider.config import Config
-from raider.plugins.basic import Cookie, Header
+from raider.plugins.basic import Cookie, File, Header
 from raider.plugins.common import Plugin
 from raider.structures import CookieStore, DataStore, HeaderStore
 from raider.user import User
@@ -65,15 +64,83 @@ class PostBody(DataStore):
         super().__init__(data)
 
 
+def process_cookies(
+    raw_cookies: CookieStore, userdata: Dict[str, str]
+) -> Dict[str, str]:
+    """Process the raw cookies and replace with the real data."""
+    cookies = raw_cookies.to_dict().copy()
+    for key in raw_cookies:
+        name = raw_cookies[key].name
+        if raw_cookies[key].name_not_known_in_advance:
+            cookies.pop(key)
+        value = raw_cookies[key].get_value(userdata)
+        if value:
+            cookies.update({name: value})
+        else:
+            cookies.pop(key)
+    return cookies
+
+
+def process_headers(
+    raw_headers: HeaderStore, userdata: Dict[str, str], config: Config
+) -> Dict[str, str]:
+    """Process the raw headers and replace with the real data."""
+    headers = raw_headers.to_dict().copy()
+    headers.update({"user-agent": config.user_agent})
+    for key in raw_headers:
+        name = raw_headers[key].name
+        if raw_headers[key].name_not_known_in_advance:
+            headers.pop(key)
+        value = raw_headers[key].get_value(userdata)
+        if value:
+            headers.update({name: value})
+        else:
+            headers.pop(name.lower())
+    return headers
+
+
+def process_data(
+    raw_data: Union[PostBody, DataStore], userdata: Dict[str, str]
+) -> Dict[str, str]:
+    """Process the raw HTTP data and replace with the real data."""
+
+    def traverse_dict(data: Dict[str, Any], userdata: Dict[str, str]) -> None:
+        """Traverse a dictionary recursively and replace plugins
+        with real data
+        """
+        for key in list(data):
+            value = data[key]
+            if isinstance(value, Plugin):
+                new_value = value.get_value(userdata)
+                if new_value:
+                    data.update({key: new_value})
+                else:
+                    data.pop(key)
+            elif isinstance(value, dict):
+                traverse_dict(value, userdata)
+
+            if isinstance(key, Plugin):
+                new_value = data.pop(key)
+                new_key = key.get_value(userdata)
+                if new_key:
+                    data.update({new_key: new_value})
+                else:
+                    data.pop(key)
+
+    httpdata = raw_data.to_dict().copy()
+    traverse_dict(httpdata, userdata)
+
+    return httpdata
+
+
 # Request needs many arguments
 # pylint: disable=too-many-arguments
 class Request:
     """Class holding the elements of the HTTP request.
 
-    When a Flow object is created, it defines a Request object with the
-    information necessary to create a HTTP request. The "method"
-    attribute is required. One and only one of "url" and "path" is
-    required too. Everything else is optional.
+    When a Flow object is created, it defines a Request object with
+    the information necessary to create a HTTP request. The "method"
+    and "url" attributes are required. Everything else is optional.
 
     The Request object can contain Plugins which will be evaluated and
     its value replaced in the HTTP request.
@@ -83,12 +150,7 @@ class Request:
         A string with the HTTP request method. Only GET and POST is
         supported for now.
       url:
-        A string with the URL of the HTTP request. Cannot be used if
-        "path" is used.
-      path:
-        A string with the path of the HTTP request. The base URL is
-        defined in the "_base_url" variable from the hy configuration
-        files of the project. If "path" is defined "url" cannot be used.
+        A string with the URL of the HTTP request.
       cookies:
         A list of Cookie objects to be sent with the HTTP request.
       headers:
@@ -107,10 +169,9 @@ class Request:
         self,
         method: str,
         url: Optional[Union[str, Plugin]] = None,
-        path: Optional[Union[str, Plugin]] = None,
         cookies: Optional[List[Cookie]] = None,
         headers: Optional[List[Header]] = None,
-        data: Optional[Union[Dict[Any, Any], PostBody]] = None,
+        data: Optional[Union[Dict[Any, Any], PostBody, File]] = None,
     ) -> None:
         """Initializes the Request object.
 
@@ -118,14 +179,7 @@ class Request:
           method:
             A string with the HTTP method. Can be either "GET" or "POST".
           url:
-            A string with the full URL of the Request. Cannot be defined
-            together with "path" argument.
-          path:
-            A string with the partial path pointing to the endpoint the
-            Request needs to be sent. This value will be prepended by
-            the "_base_url" variable set up in hy configuration files to
-            create the full URL. If "path" is defined, the "url"
-            argument cannot be defined.
+            A string with the full URL of the Request.
           cookies:
             A list of Cookie Plugins. Its values will be calculated and
             inserted into the HTTP Request on runtime.
@@ -143,14 +197,11 @@ class Request:
         if not self.method:
             logging.critical("Required :method parameter, can't run without")
 
-        if not bool(url) ^ bool(path):
-            logging.critical(
-                "One and only one of :path and :url parameters required"
-            )
+        if not url:
+            logging.critical("Required :url parameter, can't run without")
             sys.exit()
 
         self.url = url
-        self.path = path
 
         self.headers = HeaderStore(headers)
         self.cookies = CookieStore(cookies)
@@ -158,6 +209,8 @@ class Request:
         self.data: Union[PostBody, DataStore]
         if isinstance(data, PostBody):
             self.data = data
+        elif isinstance(data, File):
+            self.data = data
         else:
             self.data = DataStore(data)
 
@@ -182,9 +235,6 @@ class Request:
         if isinstance(self.url, Plugin):
             inputs.update({self.url.name: self.url})
             inputs.update(get_children_plugins(self.url))
-        if isinstance(self.path, Plugin):
-            inputs.update({self.path.name: self.path})
-            inputs.update(get_children_plugins(self.path))
 
         for name in self.cookies:
             cookie = self.cookies[name]
@@ -206,9 +256,6 @@ class Request:
 
         return inputs
 
-    # pylint: disable=W0511
-    # TODO: Will redesign this function later.
-    # pylint: disable=R0912
     def process_inputs(
         self, user: User, config: Config
     ) -> Dict[str, Dict[str, str]]:
@@ -229,47 +276,18 @@ class Request:
           from processing the inputs.
 
         """
+
         userdata = user.to_dict()
 
-        cookies = self.cookies.to_dict().copy()
-        headers = self.headers.to_dict().copy()
-        httpdata = self.data.to_dict().copy()
-
-        if self.path:
-            base_url = config.project_config["_base_url"]
-            if isinstance(self.path, Plugin):
-                path = self.path.get_value(userdata)
-            else:
-                path = self.path
-            self.url = parse.urljoin(base_url, path)
-
         if isinstance(self.url, Plugin):
             self.url = self.url.get_value(userdata)
 
-        headers.update({"user-agent": config.user_agent})
-
-        for key in self.cookies:
-            value = self.cookies[key].get_value(userdata)
-            if value:
-                cookies.update({key: value})
-
-        for key in self.headers:
-            value = self.headers[key].get_value(userdata)
-            if value:
-                headers.update({key: value})
-
-        for key in list(httpdata):
-            value = httpdata[key]
-            if isinstance(value, Plugin):
-                new_value = value.get_value(userdata)
-                if new_value:
-                    httpdata.update({key: new_value})
-
-            if isinstance(key, Plugin):
-                new_value = httpdata.pop(key)
-                new_key = key.get_value(userdata)
-                if new_key:
-                    httpdata.update({new_key: new_value})
+        cookies = process_cookies(self.cookies, userdata)
+        headers = process_headers(self.headers, userdata, config)
+        if isinstance(self.data, File):
+            httpdata = self.data.get_value(userdata)
+        else:
+            httpdata = process_data(self.data, userdata)
 
         return {"cookies": cookies, "data": httpdata, "headers": headers}
 
@@ -357,6 +375,44 @@ class Request:
                     verify=verify,
                     allow_redirects=False,
                 )
+            return req
+
+        if self.method == "PATCH":
+            if (
+                isinstance(self.data, PostBody)
+                and self.data.encoding == "json"
+            ):
+                req = requests.patch(
+                    self.url,
+                    json=data,
+                    headers=headers,
+                    cookies=cookies,
+                    proxies=proxies,
+                    verify=verify,
+                    allow_redirects=False,
+                )
+            else:
+                req = requests.patch(
+                    self.url,
+                    data=data,
+                    headers=headers,
+                    cookies=cookies,
+                    proxies=proxies,
+                    verify=verify,
+                    allow_redirects=False,
+                )
+            return req
+
+        if self.method == "PUT":
+            req = requests.put(
+                self.url,
+                data=data,
+                headers=headers,
+                cookies=cookies,
+                proxies=proxies,
+                verify=verify,
+                allow_redirects=False,
+            )
 
             return req
 
@@ -378,7 +434,6 @@ class Template(Request):
         self,
         method: str,
         url: Optional[Union[str, Plugin]] = None,
-        path: Optional[Union[str, Plugin]] = None,
         cookies: Optional[List[Cookie]] = None,
         headers: Optional[List[Header]] = None,
         data: Optional[Union[Dict[Any, Any], PostBody]] = None,
@@ -387,7 +442,6 @@ class Template(Request):
         super().__init__(
             method=method,
             url=url,
-            path=path,
             cookies=cookies,
             headers=headers,
             data=data,
@@ -397,7 +451,6 @@ class Template(Request):
         self,
         method: Optional[str] = None,
         url: Optional[Union[str, Plugin]] = None,
-        path: Optional[Union[str, Plugin]] = None,
         cookies: Optional[List[Cookie]] = None,
         headers: Optional[List[Header]] = None,
         data: Optional[Union[Dict[Any, Any], PostBody]] = None,
@@ -409,12 +462,6 @@ class Template(Request):
         of itself with the modified parameters.
 
         """
-        if bool(url) & bool(path):
-            logging.critical(
-                "One and only one of :path and :url parameters allowed"
-            )
-            sys.exit()
-
         template = deepcopy(self)
 
         if method:
@@ -423,9 +470,6 @@ class Template(Request):
         if url:
             template.url = url
 
-        if path:
-            template.path = path
-
         if cookies:
             template.cookies.merge(CookieStore(cookies))
 
